import { Chain, ChainAddress, ChainId, ChainsConfig, Contracts, NativeAddress, Network, TokenBridge, TokenId } from '@wormhole-foundation/connect-sdk';
import { AnySolanaAddress, SolanaChains, SolanaPlatformType, SolanaUnsignedTransaction } from '@wormhole-foundation/connect-sdk-solana';
import { SolanaWormholeCore } from '@wormhole-foundation/connect-sdk-solana-core';
import { Program } from '@project-serum/anchor';
import { Connection } from '@solana/web3.js';
import { Platform } from '@wormhole-foundation/sdk-base/src';
import { TokenBridge as TokenBridgeContract } from './tokenBridgeType';
import '@wormhole-foundation/connect-sdk-solana-core';
export declare class SolanaTokenBridge<N extends Network, C extends SolanaChains> implements TokenBridge<N, SolanaPlatformType, C> {
    readonly network: N;
    readonly chain: C;
    readonly connection: Connection;
    readonly contracts: Contracts;
    readonly chainId: ChainId;
    readonly coreBridge: SolanaWormholeCore<N, C>;
    readonly tokenBridge: Program<TokenBridgeContract>;
    constructor(network: N, chain: C, connection: Connection, contracts: Contracts);
    static fromRpc<N extends Network>(connection: Connection, config: ChainsConfig<N, Platform>): Promise<SolanaTokenBridge<N, SolanaChains>>;
    isWrappedAsset(token: AnySolanaAddress): Promise<boolean>;
    getOriginalAsset(token: AnySolanaAddress): Promise<TokenId>;
    hasWrappedAsset(token: TokenId): Promise<boolean>;
    getWrappedAsset(token: TokenId<Chain>): Promise<NativeAddress<C>>;
    isTransferCompleted(vaa: TokenBridge.VAA<'Transfer' | 'TransferWithPayload'>): Promise<boolean>;
    getWrappedNative(): Promise<NativeAddress<C>>;
    createAttestation(token: AnySolanaAddress, payer?: AnySolanaAddress): AsyncGenerator<SolanaUnsignedTransaction<N, C>>;
    submitAttestation(vaa: TokenBridge.VAA<'AttestMeta'>, payer?: AnySolanaAddress): AsyncGenerator<SolanaUnsignedTransaction<N, C>>;
    private transferSol;
    transfer(sender: AnySolanaAddress, recipient: ChainAddress, token: AnySolanaAddress | 'native', amount: bigint, payload?: Uint8Array): AsyncGenerator<SolanaUnsignedTransaction<N, C>>;
    private redeemAndUnwrap;
    private createAta;
    redeem(sender: AnySolanaAddress, vaa: TokenBridge.VAA<'Transfer' | 'TransferWithPayload'>, unwrapNative?: boolean): AsyncGenerator<SolanaUnsignedTransaction<N, C>, void, unknown>;
    private createUnsignedTx;
}
//# sourceMappingURL=tokenBridge.d.ts.map